#ifndef DESERIALIZE_HPP
#define DESERIALIZE_HPP

#include <list>
#include <vector>
#include <map>
#include <set>

#include "DeSerializer.hpp"
#include "Assert.hpp"

#include "Compound.hpp"

#include "CompilerConfig.hpp"

namespace PSF
{
	// DeSerialize PSF compunds
	template<class t_TypeList,class t_Unique,class t_Base>
	class TCompoundDeSerializer
	{
	public:
		typedef TCompound<t_TypeList,t_Unique,t_Base> Compound;

		TCompoundDeSerializer( Compound& i_compound) :
			m_compound( i_compound)
		{
		}

		bool DeSerialize( IDeSerializer& i_deserializer)
		{
			TInner<Compound> inner( m_compound);
			return inner.DeSerialize( i_deserializer);
		}

	private:
		template<class t_InnerCompund>
		class TInner
		{
		public:
			TInner( t_InnerCompund& i_innerCompound) : 
				m_innerCompound( i_innerCompound)
			{
			}

			bool DeSerialize( IDeSerializer& i_deserializer)
			{
				TInner<t_InnerCompund::Base> inner( m_innerCompound);
				bool bResult = inner.DeSerialize( i_deserializer);
				return bResult && DeSerializeInternal<0>( i_deserializer);
			}
		private:

			enum { number_of_loops = t_InnerCompund::number_of_members-1};

			template<int t_i>
			bool DeSerializeInternal( IDeSerializer& i_deserializer)
			{
				PSF_ASSERT( t_InnerCompund::ms_namelist[t_i]);
				const wchar_t* pwszName = t_InnerCompund::ms_namelist[t_i];
				bool bResult = PSF::DeSerialize( m_innerCompound.Member<t_i>(), i_deserializer, pwszName);
				return bResult && DeSerializeInternal<t_i+1>( i_deserializer);
			}

			template<>
			bool DeSerializeInternal<number_of_loops>( IDeSerializer& i_deserializer)
			{
				PSF_ASSERT( t_InnerCompund::ms_namelist[number_of_loops]);
				const wchar_t* pwszName = t_InnerCompund::ms_namelist[number_of_loops];
				return PSF::DeSerialize( m_innerCompound.Member<number_of_loops>(), i_deserializer, pwszName);
			}

			t_InnerCompund& m_innerCompound;
		};

		template<>
		class TInner<Loki::EmptyType>
		{
		public:
			TInner<Loki::EmptyType>( const t_InnerCompund& i_innerCompound)
			{
			}

			bool DeSerialize( IDeSerializer& i_deserializer) const
			{
				return true;
			}
		};

		Compound& m_compound;
	};

	template<class t_TypeList,class t_Unique,class t_Base>
	bool DeSerialize( TCompound<t_TypeList,t_Unique,t_Base>& o_compound, 
					IDeSerializer& i_deserializer, 
					const std::wstring& i_wstrName = L"Compound")
	{
		std::auto_ptr<IDeSerializer> pDeSerializer( i_deserializer.DeSerializeCompound( i_wstrName, typeid(t_Unique)));
		if ( !pDeSerializer.get()) return false;
		TCompoundDeSerializer<t_TypeList,t_Unique,t_Base> compoundDeSerializer( o_compound);
		return compoundDeSerializer.DeSerialize( *pDeSerializer);
	}
};

namespace PSF
{
	template<class t_iteratorType>
	bool DeSerializeRange( t_iteratorType o_iter, 
						IDeSerializer& i_deserializer, 
						const std::wstring& i_wstrName, 
						const Loki::TypeInfo& i_typeInfo)
	{
		std::auto_ptr<IDeSerializer> pDeSerializer( i_deserializer.DeSerializeRange( i_wstrName, i_typeInfo));
		if ( !pDeSerializer.get()) return false;

		t_iteratorType::container_type::value_type val;
		while ( DeSerialize( val, *pDeSerializer))
		{
			*o_iter = val;
			o_iter++;
		}
		return true;
	}


	// DeSerializers for common containers
	template<typename t_type>
	bool DeSerialize( std::basic_string<t_type>& o_out, IDeSerializer& i_deserializer, const std::wstring& i_wstrName = L"String")
	{
		std::back_insert_iterator<std::basic_string<t_type> > iter( o_out);
		return DeSerializeRange( iter, i_deserializer, i_wstrName, Loki::TypeInfo(typeid(std::basic_string<void>)));
	}

	template<typename t_type>
	bool DeSerialize( std::list<t_type>& o_out, IDeSerializer& i_deserializer, const std::wstring& i_wstrName = L"List")
	{
		std::back_insert_iterator<std::list<t_type> > iter( o_out);
		return DeSerializeRange( iter, i_deserializer, i_wstrName, Loki::TypeInfo(typeid(std::list<void>)));
	}

	template<typename t_type>
	bool DeSerialize( std::vector<t_type>& o_out, IDeSerializer& i_deserializer, const std::wstring& i_wstrName = L"Vector")
	{
		std::back_insert_iterator<std::vector<t_type> > iter( o_out);
		return DeSerializeRange( iter, i_deserializer, i_wstrName, Loki::TypeInfo(typeid(std::vector<void>)));
	}

	template<typename t_type>
	bool DeSerialize( std::set<t_type>& o_out, IDeSerializer& i_deserializer, const std::wstring& i_wstrName = L"Set")
	{
		std::insert_iterator<std::set<t_type> > iter( o_out, o_out.end());
		return DeSerializeRange( iter, i_deserializer, i_wstrName, Loki::TypeInfo(typeid(std::set<void>)));
	}

	template<typename T>
	T& UnConst( const T& i_in)
	{
		return const_cast<T&>( i_in);
	}

	template<typename t_typeFirst,typename t_typeSecond>
	bool DeSerialize( std::pair<t_typeFirst,t_typeSecond>& o_out, IDeSerializer& i_deserializer, const std::wstring& i_wstrName = L"Pair")
	{
		std::auto_ptr<IDeSerializer> pDeSerializer( 
			i_deserializer.DeSerializeCompound( i_wstrName, Loki::TypeInfo(typeid(std::pair<void,void>))));
		if ( !pDeSerializer.get()) return false;

		// Should be compile-time const removal, but sadly that does not seem
		// to be possible i VC7
		return DeSerialize( UnConst(o_out.first), *pDeSerializer, L"First") && 
			DeSerialize( o_out.second, *pDeSerializer, L"Second");
	}

	template<typename t_key,typename t_type>
	bool DeSerialize( std::map<t_key,t_type>& o_out, IDeSerializer& i_deserializer, const std::wstring& i_wstrName = L"Map")
	{
		std::insert_iterator<std::map<t_key,t_type> > iter( o_out, o_out.end());
		return DeSerializeRange( iter, i_deserializer, i_wstrName, Loki::TypeInfo(typeid(std::map<void,void>)));
	}

	// POD DeSerializers
	#ifdef PSF_PARTIAL_ORDERING_OF_TEMPLATES
	// Template serializers
	template<typename t_type>
	bool PSF::DeSerialize( t_type& o_out, IDeSerializer& i_deserializer)
	{
		return i_deserializer.DeSerialize( o_out);
	}

	template<typename t_type>
	bool PSF::DeSerialize( t_type& o_out, IDeSerializer& i_deserializer, const std::wstring& i_wstrName)
	{
		std::auto_ptr<IDeSerializer> pDeSerializer( i_deserializer.DeSerializeCompound( i_wstrName, Loki::TypeInfo(typeid(t_type))));
		if ( !pDeSerializer.get()) return false;
		return DeSerialize( o_out, *pDeSerializer);
	}
	#else
	// Explicit deserializers (for compilers that does not support partial ordering of templates)
	bool DeSerialize( char& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( unsigned char& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( short& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( unsigned short& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( int& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( unsigned int& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( long& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( unsigned long& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( float& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( double& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( bool& i_in, IDeSerializer& i_deserializer);
	bool DeSerialize( wchar_t& i_in, IDeSerializer& i_deserializer);

	bool DeSerialize( char& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( unsigned char& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( short& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( unsigned short& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( int& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( unsigned int& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( long& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( unsigned long& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( float& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( double& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( bool& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	bool DeSerialize( wchar_t& i_in, IDeSerializer& i_deserializer, const std::wstring& i_wstrName);
	#endif
}

#endif