#ifndef PSF_SERIALIZE_HPP
#define PSF_SERIALIZE_HPP

#include <list>
#include <vector>
#include <map>
#include <set>

#include "Serializer.hpp"
#include "Assert.hpp"

#include "Compound.hpp"

#include "CompilerConfig.hpp"

namespace PSF
{
	// Serialize PSF compunds
	template<class t_TypeList,class t_Unique,class t_Base>
	class TCompoundSerializer
	{
	public:
		typedef TCompound<t_TypeList,t_Unique,t_Base> Compound;

		TCompoundSerializer( const Compound& i_compound) :
			m_compound( i_compound)
		{
		}

		void Serialize( ISerializer& i_serializer) const
		{
			TInner<Compound> inner( m_compound);
			inner.Serialize( i_serializer);
		}

	private:
		template<class t_InnerCompund>
		class TInner
		{
		public:
			TInner( const t_InnerCompund& i_innerCompound) : 
				m_innerCompound( i_innerCompound)
			{
			}

			void Serialize( ISerializer& i_serializer) const
			{
				TInner<t_InnerCompund::Base> inner( m_innerCompound);
				inner.Serialize( i_serializer);
				SerializeInternal<0>( i_serializer);
			}
		private:

			enum { number_of_loops = t_InnerCompund::number_of_members-1};

			template<int t_i>
			void SerializeInternal( ISerializer& i_serializer) const
			{
				PSF_ASSERT( t_InnerCompund::ms_namelist[t_i]);
				const wchar_t* pwszName = t_InnerCompund::ms_namelist[t_i];
				PSF::Serialize( m_innerCompound.Member<t_i>(), i_serializer, pwszName);
				SerializeInternal<t_i+1>( i_serializer);
			}

			template<>
			void SerializeInternal<number_of_loops>( ISerializer& i_serializer) const
			{
				PSF_ASSERT( t_InnerCompund::ms_namelist[number_of_loops]);
				const wchar_t* pwszName = t_InnerCompund::ms_namelist[number_of_loops];
				PSF::Serialize( m_innerCompound.Member<number_of_loops>(), i_serializer, pwszName);
			}
			
			const t_InnerCompund& m_innerCompound;
		};

		template<>
		class TInner<Loki::EmptyType>
		{
		public:
			TInner<Loki::EmptyType>( const t_InnerCompund& i_innerCompound)
			{
			}

			void Serialize( ISerializer& i_serializer) const
			{
			}
		};

		const Compound& m_compound;
	};

	template<class t_TypeList,class t_Unique,class t_Base>
	void Serialize( const TCompound<t_TypeList,t_Unique,t_Base>& i_compound, 
					ISerializer& i_serializer, 
					const std::wstring& i_wstrName = L"Compound")
	{
		std::auto_ptr<ISerializer> pSerializer( i_serializer.SerializeCompound( i_wstrName, typeid( t_Unique)));
		TCompoundSerializer<t_TypeList,t_Unique,t_Base> compoundSerializer( i_compound);
		compoundSerializer.Serialize( *pSerializer);
	}
};

namespace PSF
{
	template<class t_iterType>
	void SerializeRange( t_iterType i_iterBegin, t_iterType i_iterEnd, 
						ISerializer& i_serializer, 
						const std::wstring& i_wstrName, 
						const Loki::TypeInfo& i_typeInfo)
	{
		std::auto_ptr<ISerializer> pSerializer( i_serializer.SerializeRange( i_wstrName, i_typeInfo));
		for ( ; i_iterBegin != i_iterEnd; ++i_iterBegin)
		{
			Serialize( *i_iterBegin, *pSerializer);
		}
	}

	// Serializers for common containers
	template<typename t_type>
	void Serialize( const std::basic_string<t_type>& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName = L"String")
	{
		SerializeRange( i_in.begin(), i_in.end(), i_serializer, i_wstrName, Loki::TypeInfo(typeid(std::basic_string<void>)));
	}

	template<typename t_type>
	void Serialize( const std::list<t_type>& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName = L"List")
	{
		SerializeRange( i_in.begin(), i_in.end(), i_serializer, i_wstrName, Loki::TypeInfo(typeid(std::list<void>)));
	}

	template<typename t_type>
	void Serialize( const std::vector<t_type>& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName = L"Vector")
	{
		SerializeRange( i_in.begin(), i_in.end(), i_serializer, i_wstrName, Loki::TypeInfo(typeid(std::vector<void>)));
	}

	template<typename t_type>
	void Serialize( const std::set<t_type>& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName = L"Set")
	{
		SerializeRange( i_in.begin(), i_in.end(), i_serializer, i_wstrName, Loki::TypeInfo(typeid(std::set<void>)));
	}

	template<typename t_typeFirst,class t_typeSecond>
	void Serialize( const std::pair<t_typeFirst,t_typeSecond>& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName = L"Pair")
	{
		std::auto_ptr<ISerializer> pSerializer( i_serializer.SerializeCompound( i_wstrName, Loki::TypeInfo(typeid(std::pair<void,void>))));
		Serialize( i_in.first, *pSerializer, L"First");
		Serialize( i_in.second, *pSerializer, L"Second");
	}

	template<typename t_key,typename t_type>
	void Serialize( const std::map<t_key,t_type>& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName = L"Map")
	{
		SerializeRange( i_in.begin(), i_in.end(), i_serializer, i_wstrName, Loki::TypeInfo(typeid(std::map<void,void>)));
	}

	// POD Serializers
	#ifdef PSF_PARTIAL_ORDERING_OF_TEMPLATES
	// Template serializers
	template<typename t_type>
	void PSF::Serialize( t_type& i_in, ISerializer& i_serializer)
	{
		i_serializer.Serialize( i_in);
	}

	template<typename t_type>
	void PSF::Serialize( t_type& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName)
	{
		Serialize( i_in, *i_serializer.SerializeCompound( i_wstrName, Loki::TypeInfo(typeid(t_type))));
	}
	#else
	// Explicit serializers (for compilers that does not support partial ordering of templates)
	void Serialize( const char& i_in, ISerializer& i_serializer);
	void Serialize( const unsigned char& i_in, ISerializer& i_serializer);
	void Serialize( const short& i_in, ISerializer& i_serializer);
	void Serialize( const unsigned short& i_in, ISerializer& i_serializer);
	void Serialize( const int& i_in, ISerializer& i_serializer);
	void Serialize( const unsigned int& i_in, ISerializer& i_serializer);
	void Serialize( const long& i_in, ISerializer& i_serializer);
	void Serialize( const unsigned long& i_in, ISerializer& i_serializer);
	void Serialize( const float& i_in, ISerializer& i_serializer);
	void Serialize( const double& i_in, ISerializer& i_serializer);
	void Serialize( const bool& i_in, ISerializer& i_serializer);
	void Serialize( const wchar_t& i_in, ISerializer& i_serializer);

	void Serialize( const char& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const unsigned char& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const short& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const unsigned short& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const int& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const unsigned int& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const long& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const unsigned long& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const float& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const double& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const bool& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	void Serialize( const wchar_t& i_in, ISerializer& i_serializer, const std::wstring& i_wstrName);
	#endif
};

#endif