#ifndef _UTILITY_STRING_H_
#define _UTILITY_STRING_H_

#include <string>
#include <sstream>
#include <algorithm>
#include <iterator>
#include <locale>
#include <ostream>


namespace Utility
{
	class CCharWCharConvert
	{
	public:
		CCharWCharConvert() :
			m_ctype( std::use_facet<std::ctype<wchar_t> > ( std::locale::locale()))
		{
		}

		CCharWCharConvert( std::locale& i_locale) :
			m_ctype( std::use_facet<std::ctype<wchar_t> > ( i_locale))
		{
		}

		char operator() ( wchar_t i_wc)
		{
			return m_ctype.narrow( i_wc);
		}

		wchar_t operator() ( char i_c)
		{
			return m_ctype.widen( i_c);
		}

	private:
		const std::ctype<wchar_t>& m_ctype;

	};

	template< class t_CharType>
	class CCharacterToLower
	{
	public:
		CCharacterToLower() :
			m_ctype( std::use_facet<std::ctype<t_CharType> > ( std::locale::locale()))
		{
		}

		CCharacterToLower( std::locale& i_locale) :
			m_ctype( std::use_facet<std::ctype<t_CharType> > ( i_locale))
		{
		}

		void operator() ( t_CharType& i_char)
		{
			i_char = m_ctype.tolower( i_char);
		}

	private:
		const std::ctype<t_CharType>& m_ctype;

	};

	template< class t_CharType>
	class CCharacterToUpper
	{
	public:
		CCharacterToUpper() :
			m_ctype( std::use_facet<std::ctype<t_CharType> > ( std::locale::locale()))
		{
		}

		CCharacterToUpper( std::locale& i_locale) :
			m_ctype( std::use_facet<std::ctype<t_CharType> > ( i_locale))
		{
		}

		void operator() ( t_CharType& i_char)
		{
			i_char = m_ctype.toupper( i_char);
		}

	private:
		const std::ctype<t_CharType>& m_ctype;

	};

	class CIsWhitespace
	{
	public:
		CIsWhitespace() :
			m_locale( std::locale::locale())
		{
		}

		CIsWhitespace( std::locale& i_locale) :
			m_locale( i_locale)
		{
		}

		template< class t_CharType>
		bool operator() ( t_CharType i_char)
		{
			return !std::isgraph( i_char, m_locale);
		}

	private:
		const std::locale& m_locale;
	};

	class CIsNotWhitespace
	{
	public:
		CIsNotWhitespace() :
			m_locale( std::locale::locale())
		{
		}

		CIsNotWhitespace( std::locale& i_locale) :
			m_locale( i_locale)
		{
		}

		template< class t_CharType>
		bool operator() ( t_CharType i_char)
		{
			return std::isgraph( i_char, m_locale);
		}

	private:
		std::locale m_locale;
	};


	template< class t_InputIterator, class t_OutputIterator>
	void CharWCharConvert( t_InputIterator i_iterBegin, t_InputIterator i_iterEnd, t_OutputIterator& o_iter)
	{
		std::transform( i_iterBegin, i_iterEnd, o_iter, CCharWCharConvert());
	}	

	template< class t_InputIterator>
	void CharToLower( t_InputIterator i_iterBegin, t_InputIterator i_iterEnd)
	{
		std::for_each( i_iterBegin, i_iterEnd, CCharacterToLower<char>());
	}

	template< class t_InputIterator>
	void WCharToLower( t_InputIterator i_iterBegin, t_InputIterator i_iterEnd)
	{
		std::for_each( i_iterBegin, i_iterEnd, CCharacterToLower<wchar_t>());
	}

	template< class t_InputIterator>
	void CharToUpper( t_InputIterator i_iterBegin, t_InputIterator i_iterEnd)
	{
		std::for_each( i_iterBegin, i_iterEnd, CCharacterToUpper<char>());
	}

	template< class t_InputIterator>
	void WCharToUpper( t_InputIterator i_iterBegin, t_InputIterator i_iterEnd)
	{
		std::for_each( i_iterBegin, i_iterEnd, CCharacterToUpper<wchar_t>());
	}

	template< class t_InputIterator>
	t_InputIterator SkipWhiteSpace( t_InputIterator i_iterBegin, t_InputIterator i_iterEnd)
	{
		return std::find_if( i_iterBegin, i_iterEnd, CIsNotWhitespace());
	}

	template< class t_type>
	std::wstring ValueToString( const t_type& i_in)
	{
		std::wstringstream wstrstream;
		wstrstream << std::boolalpha << i_in;
		return wstrstream.str();
	}

	template< class t_type>
	void StringToValue( const std::wstring& i_in, t_type& o_out)
	{
		std::wstringstream wstrStream( i_in);
		wstrStream >> std::boolalpha >> o_out;
	}

	template< class t_type>
	void StringToValue( const std::string& i_in, t_type& o_out)
	{
		std::stringstream strStream( i_in);
		strStream >> std::boolalpha >> o_out;
	}

	/*
	template< class t_string, class t_OutputIterator>
	void Replace( const t_string& i_str, t_string i_strPattern, t_string i_strNew, t_string& o_str)
	{
		t_string::size_type nOffset = 0;
		while ( (nOffset != i_str::npos) && (nOffset < i_str.size()))
		{
			t_string::size_type nOldOffset = nOffset;
			nOffset = i_str.find_first_of( i_strPattern);
			if ( nOffset != i_str::npos)
			{
				o_str += i_str.substr( nOldOffset, nOffset - nOldOffset);
				o_str += i_strNew;
				nOffset += i_strPattern.size();
			}
			else
			{
				o_str += i_str.substr( nOldOffset);
			}
		}
	}*/

	/*
	void FormatDateString(unsigned int i_nDate, std::string& o_strOut);
	void FormatDateString(unsigned int i_nDate, std::wstring& o_wstrOut);

	void FormatDateStringHdr(unsigned int i_nDate, std::string& o_strOut);
	void FormatDateStringHdr(unsigned int i_nDate, std::wstring& o_wstrOut);
*/
}

std::wostream& operator << ( std::wostream& i_wostream, const std::string& i_str);
std::ostream& operator << ( std::ostream& i_ostream, const std::wstring& i_wstr);
#endif