#include "XMLSerializer.hpp"

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/framework/LocalFileFormatTarget.hpp>
XERCES_CPP_NAMESPACE_USE

#include <exception>
#include <sstream>
#include <iostream>

template<class t_type>
void SerializePOD( const t_type& i_in, std::wstringstream& i_wstrstream, wchar_t i_wcDelimiter = L' ')
{
	i_wstrstream << std::boolalpha << i_in;
	if ( i_wcDelimiter) i_wstrstream << i_wcDelimiter;
}

//////////////////////////////////////////////////////////////////////////////////////
// CXMLSerializerBase : Implmentation
//////////////////////////////////////////////////////////////////////////////////////

PSF::CXMLSerializerBase::CXMLSerializerBase( DOMDocument* i_pdomDoc, 
										DOMElement* i_pdomElementRoot, 
										const SerializerTypeMap& i_mapType,
										bool i_bRange) :
	m_pdomDoc( i_pdomDoc),
	m_pdomElementRoot( i_pdomElementRoot),
	m_mapType( i_mapType),
	m_bRange( i_bRange),
	m_nItemsSerialized( 0)
{
}

PSF::CXMLSerializerBase::~CXMLSerializerBase()
{
	XERCES_CPP_NAMESPACE::DOMText* pdomText = m_pdomDoc->createTextNode( m_wstrstream.str().c_str());
	m_pdomElementRoot->appendChild( pdomText);

	if ( m_bRange)
	{
		std::wstringstream wstrstream;
		wstrstream << m_nItemsSerialized;
		m_pdomElementRoot->setAttribute( L"size", wstrstream.str().c_str());
	}
}

void PSF::CXMLSerializerBase::Serialize( const char& i_in)
{
	SerializePOD( i_in, m_wstrstream, 0);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const unsigned char& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const short& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const unsigned short& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const int& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const unsigned int& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const long& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const unsigned long& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const float& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const double& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const wchar_t& i_in)
{
	SerializePOD( i_in, m_wstrstream, 0);
	m_nItemsSerialized++;
}

void PSF::CXMLSerializerBase::Serialize( const bool& i_in)
{
	SerializePOD( i_in, m_wstrstream);
	m_nItemsSerialized++;
}

std::auto_ptr<PSF::ISerializer> PSF::CXMLSerializerBase::SerializeCompound( const std::wstring& i_wstrName, 
																		    const Loki::TypeInfo& i_type)
{
	m_nItemsSerialized++;
	DOMElement* pdomElement = m_pdomDoc->createElement( i_wstrName.c_str());
	m_pdomElementRoot->appendChild( pdomElement);
	SerializerTypeMap::const_iterator iterType = m_mapType.find( i_type);
	if ( m_mapType.end() != iterType)
	{
		pdomElement->setAttribute( L"type", iterType->second.c_str());
	}

	return std::auto_ptr<ISerializer>( new CXMLSerializerBase( m_pdomDoc, pdomElement, m_mapType, false));
}

std::auto_ptr<PSF::ISerializer> PSF::CXMLSerializerBase::SerializeRange( const std::wstring& i_wstrName, 
																	     const Loki::TypeInfo& i_type)
{
	m_nItemsSerialized++;
	DOMElement* pdomElement = m_pdomDoc->createElement( i_wstrName.c_str());
	m_pdomElementRoot->appendChild( pdomElement);
	SerializerTypeMap::const_iterator iterType = m_mapType.find( i_type);
	if ( m_mapType.end() != iterType)
	{
		pdomElement->setAttribute( L"type", iterType->second.c_str());
	}

	return std::auto_ptr<ISerializer>( new CXMLSerializerBase( m_pdomDoc, pdomElement, m_mapType, true));
}

//////////////////////////////////////////////////////////////////////////////////////
// CXMLSerializerProxy : Implmentation
//////////////////////////////////////////////////////////////////////////////////////

PSF::CXMLSerializerProxy::CXMLSerializerProxy( const std::wstring& i_wstrFilename, 
							    const std::wstring& i_wstrRoot, 
							    const SerializerTypeMap& i_mapType) :
	m_wstrFilename( i_wstrFilename),
	m_pdomImpl( 0),
	m_pdomDoc( 0)
{
	try 
	{
		XMLPlatformUtils::Initialize();
		m_pdomImpl = (DOMImplementation*)DOMImplementationRegistry::getDOMImplementation( L"Core XML 1.0 LS");
		m_pdomDoc = m_pdomImpl->createDocument( 0, i_wstrRoot.c_str(), 0);
		XERCES_CPP_NAMESPACE::DOMElement* pdomElementRoot = m_pdomDoc->getDocumentElement();
		m_pBase = std::auto_ptr<CXMLSerializerBase>( new CXMLSerializerBase( m_pdomDoc, pdomElementRoot, i_mapType, false));
	}
	catch (const XMLException& toCatch) 
	{
		char* pszMessage = XMLString::transcode( toCatch.getMessage());
		throw std::exception( pszMessage);
	}
}

PSF::CXMLSerializerProxy::~CXMLSerializerProxy()
{
	DOMWriter* pdomWriter = m_pdomImpl->createDOMWriter();

	if ( pdomWriter->canSetFeature(XMLUni::fgDOMWRTDiscardDefaultContent, true))
	{
		pdomWriter->setFeature(XMLUni::fgDOMWRTDiscardDefaultContent, true);
	}

	if ( pdomWriter->canSetFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true))
	{
		pdomWriter->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true);
	}

	{
		LocalFileFormatTarget formatTarget( m_wstrFilename.c_str());
		pdomWriter->writeNode( &formatTarget, *m_pdomDoc);
	}

    XMLPlatformUtils::Terminate();
}

PSF::CXMLSerializerProxy::operator PSF::ISerializer&()
{
	return *m_pBase;
}
